@@ -4,7 +4,7 @@
  * Distributed under the terms of the GNU General Public License, v2 or later 
  * Author: Brad House <brad@mainstreetsoftworks.com>
  *    
- * $Header: /var/cvsroot/gentoo-src/sandbox/sandbox.h,v 1.6 2005/05/03 21:43:50 azarah Exp $
+ * $Header: /var/cvsroot/gentoo-src/sandbox/sandbox.h,v 1.14 2005/05/13 15:23:28 azarah Exp $
  */
 
 #ifndef __SANDBOX_H__
@@ -16,24 +16,23 @@
 /* Uncomment below to use flock instead of fcntl (POSIX way) to lock/unlock files */
 /* #define USE_FLOCK */
 
-/* Uncomment below to use system() to execute the shell rather than execv */
-/* #define USE_SYSTEM_SHELL */
-
-/* Uncomment below to use /etc/ld.so.preload (could be very intrusive!!) */
-/* #define USE_LD_SO_PRELOAD */
-
-/* Uncommend to not have the protected shell forked, just run in parent process */
-/* ONLY FOR DEBUGGING PURPOSES!! (strace needs it like that) */
-/* #define NO_FORK */
-
+#define LD_PRELOAD_EQ          "LD_PRELOAD="
 #define LD_PRELOAD_FILE        "/etc/ld.so.preload"
 #define LIB_NAME               "libsandbox.so"
 #define BASHRC_NAME            "sandbox.bashrc"
-#define PIDS_FILE              "/tmp/sandboxpids.tmp"
-#define LOG_FILE_PREFIX        "/tmp/sandbox-"
-#define DEBUG_LOG_FILE_PREFIX  "/tmp/sandbox-debug-"
+#define TMPDIR                 "/tmp"
+#define VAR_TMPDIR             "/var/tmp"
+#define PORTAGE_TMPDIR         "/var/tmp/portage"
+#define PIDS_FILE              "/sandboxpids.tmp"
+#define LOG_FILE_PREFIX        "/sandbox-"
+#define DEBUG_LOG_FILE_PREFIX  "/sandbox-debug-"
 #define LOG_FILE_EXT           ".log"
 
+#define ENV_LD_PRELOAD         "LD_PRELOAD"
+
+#define ENV_TMPDIR             "TMPDIR"
+#define ENV_PORTAGE_TMPDIR     "PORTAGE_TMPDIR"
+
 #define ENV_SANDBOX_DEBUG_LOG  "SANDBOX_DEBUG_LOG"
 #define ENV_SANDBOX_LOG        "SANDBOX_LOG"
 #define ENV_SANDBOX_DIR        "SANDBOX_DIR"
@@ -50,15 +49,19 @@
 
 #define DEFAULT_BEEP_COUNT     3
 
+#define SB_BUF_LEN             2048
+
 #if !HAVE_DLVSYM
 # define dlvsym(lib, sym, ver) dlsym(lib, sym)
 #endif
 
 SB_STATIC char *get_sandbox_path(char *argv0);
 SB_STATIC char *get_sandbox_lib(char *sb_path);
-SB_STATIC char *get_sandbox_pids_file(void);
+SB_STATIC char *get_sandbox_pids_file(const char *tmp_dir);
 SB_STATIC char *get_sandbox_rc(char *sb_path);
-SB_STATIC char *get_sandbox_log();
+SB_STATIC char *get_sandbox_log(const char *tmp_dir);
+SB_STATIC char *get_sandbox_debug_log(const char *tmp_dir);
+SB_STATIC int get_tmp_dir(char *tmp_dir);
 SB_STATIC char *sb_dirname(const char *path);
 SB_STATIC int file_getmode(char *mode);
 SB_STATIC long file_tell(int fp);
